// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>
#include <gsl/gsl_multimin.h>
#include "fit.h"

double get_tail_error(struct fitvars *config)
{
double Ee=0.0;
double Eh=0.0;
double ret=0.0;
double Nc=0.0;
double Nv=0.0;
double mue=0.0;
double muh=0.0;
double pl=0.0;
double nr=0.0;
double lumo_sigma=0.0;
double homo_sigma=0.0;
double Eg=0.0;

FILE *in;
char command[100];
char file1[100];
char file2[100];
int line1;
int line2;
double min;
double max;
double adderror;
double value1;
double value2;

in=fopen("constraints.inp","r");
do
{
	fscanf(in,"%s",command);
	if (strcmp(command,"#end")==0)
	{
		break;
	}else
	if (strcmp(command,"mm")==0)
	{
		fscanf(in,"%s",file1);
		fscanf(in,"%d",&line1);
		fscanf(in,"%le",&max);
		fscanf(in,"%le",&min);
		fscanf(in,"%le",&adderror);
		value1=fabs(read_value(file1,0,line1));
		printf("mm file=%s line=%d max=%le min=%le addeeor=%le\n",file1,line1,max,min,adderror);
		if (fabs(value1)<fabs(min))
		{
			ret+=adderror;
			printf("Smaller than min\n");
		}

		if (fabs(value1)>fabs(max))
		{
			ret+=adderror;
			printf("Bigger than max\n");
		}
	}else
	if (strcmp(command,"bs")==0)
	{
		fscanf(in,"%s",file1);
		fscanf(in,"%d",&line1);
		fscanf(in,"%s",file2);
		fscanf(in,"%d",&line2);
		fscanf(in,"%le",&adderror);
		printf("bs file1=%s line=%ld file2=%s line2=%ld adderror=%le\n",file1,line1,file2,line2,adderror);

		value1=fabs(read_value(file1,0,line1));
		value2=fabs(read_value(file2,0,line2));
		if (value1<value2)
		{
			printf("Wrong way arround\n");
			ret+=adderror;
		}
	}else
	{
		printf("Command not recognized\n");
		exit(0);
	}

}while(!feof(in));

if (strcmp(command,"#end")!=0)
{
	printf("Error at end of constraints file\n");
	exit(0);
}

fclose(in);


/*if (config->fit_light2_iv==TRUE)
{
	double a=0.0;
	double b=0.0;
	double y0=0.0;
	double y1=0.0;
	FILE *in;
	FILE *out=fopen("./sim/voc_tau.dat","w");
	in=fopen("./sim/ivlight/kvoc.dat","r");
	fscanf(in,"%le %le",&a,&b);
	fprintf(out,"%le %le\n",a,b);
	fclose(in);
	y0=b;
	in=fopen("./sim/ivlight2/kvoc.dat","r");
	fscanf(in,"%le %le",&a,&b);
	fprintf(out,"%le %le\n",a,b);
	fclose(in);
	y1=b;
	fclose(out);
	if ((y1-y0)<1e-20) ret+=100.0;

	out=fopen("./sim/voc_tau.dat","w");

	in=fopen("./sim/ivlight/voc_tau.dat","r");
	fscanf(in,"%le %le",&a,&b);
	fprintf(out,"%le %le\n",a,b);
	fclose(in);

	in=fopen("./sim/ivlight2/voc_tau.dat","r");
	fscanf(in,"%le %le",&a,&b);
	fprintf(out,"%le %le\n",a,b);
	fclose(in);

	fclose(out);

	//printf("%le\n",y1-y0);
	//getchar();
}*/

return ret;
}


double get_voc_tau_error(struct fitvars *config)
{
double a,b;
printf("get_tau_error\n");

FILE *in;
FILE *out;

out=fopen("./sim/voc_tau.dat","w");

in=fopen("./sim/jvlight/voc_tau.dat","r");
fscanf(in,"%le %le",&a,&b);
fprintf(out,"%le %le\n",a,b);
fclose(in);

in=fopen("./sim/jvlight2/voc_tau.dat","r");
fscanf(in,"%le %le",&a,&b);
fprintf(out,"%le %le\n",a,b);
fclose(in);

fclose(out);


char expfile[1000];
char simfile[1000];
char out_name[1000];
strcpy(expfile,"./exp/voc_tau/voc_tau.dat");
strcpy(simfile,"./sim/voc_tau.dat");
sprintf(out_name,"./error_voc_tau.dat");

double error=0.0;
struct istruct exp;
struct istruct sim;
inter_load_a(&exp,expfile,0,1);
inter_load_a(&sim,simfile,0,1);
inter_sort(&sim);

double start=0.0;
double stop=0.0;
char tempstring[100];
double max=0.0;

	FILE *info=fopena("./exp/voc_tau/","fit.inp","r");
	if (info==NULL)
	{
		printf("Problem with ./exp/voc_tau/fit.inp\n");
		exit(0);
	}
	fscanf(info,"%s",tempstring);
	fscanf(info,"%le",&start);
	fscanf(info,"%s",tempstring);
	fscanf(info,"%le",&stop);
	fscanf(info,"%s",tempstring);

	if (strcmp("#end",tempstring)!=0)
	{
		printf("Problem with fit.inp in %s!\n","./exp/voc_tau/");
		exit(0);
	}
	fclose(info);
start=exp.data[0];
if (sim.x[0]>start) start=sim.x[0];
if (exp.x[0]>start) start=exp.x[0];


inter_log_y(&sim);
inter_log_y(&exp);




double min=exp.data[exp.len-1];

inter_sub(&exp,min);
inter_sub(&sim,min);




max=exp.data[0];//inter_get(&exp,start);//inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&exp,1.0, 1.0/max);
inter_rescale(&sim,1.0, 1.0/max);


error=inter_get_diff(&exp,&sim,out_name,start,stop,config->voc_tau_mul);

inter_free(&exp);
inter_free(&sim);
return error;
}

double get_charge_light_ss_error(struct fitvars *config)
{
printf("get ce light ss error\n");
double error=0.0;
double max=0.0;
char exp_name[40];
char sim_name[40];
char out_name[40];
strcpy(exp_name,"./exp/celight/ce.dat");
strcpy(sim_name,"./sim/jvlight/charge.dat");
strcpy(out_name,"./error_ce_light.dat");

struct istruct exp;
struct istruct sim;

inter_load_a(&exp,exp_name,0,1);
inter_load_a(&sim,sim_name,0,1);


inter_log_y(&sim);
inter_log_y(&exp);

double start=exp.x[1];
if (sim.x[0]>start) start=sim.x[0];
start=0.3;
double stop=0.6;

double min=inter_get(&exp,start);
inter_sub(&exp,min);
inter_sub(&sim,min);


max=inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&sim,1.0, 1.0/max);


FILE *ferror;
ferror=fopen(out_name,"w");


error=inter_get_diff(&exp,&sim,out_name,start,stop,config->chargedark_fitmul);

fclose(ferror);

inter_free(&exp);
inter_free(&sim);

return error;
}


double get_jv_error_light(struct fitvars *config)
{
double error=0.0;
char tempstring[1000];
struct istruct exp;
struct istruct sim;

char exp_name[40];
char sim_name[40];
char out_name[40];
strcpy(exp_name,"./exp/jvlight/jv.dat");
strcpy(sim_name,"./sim/jvlight/jvexternal.dat");
strcpy(out_name,"./error_jv_light.dat");

inter_load_a(&exp,exp_name,0,1);
inter_load_a(&sim,sim_name,0,1);

inter_save(&exp,"./a.dat");
inter_save(&sim,"./b.dat");

double start=0.3;
double stop=0.7;

	FILE *info=fopena("./exp/jvlight/","fit.inp","r");
	if (info==NULL)
	{
		printf("Problem with ./exp/jvlight/fit.inp\n");
		exit(0);
	}
	fscanf(info,"%s",tempstring);
	fscanf(info,"%le",&start);
	fscanf(info,"%s",tempstring);
	fscanf(info,"%le",&stop);
	fscanf(info,"%s",tempstring);

	if (strcmp("#end",tempstring)!=0)
	{
		printf("Problem with fit.inp in %s!\n","./exp/jvlight/");
		exit(0);
	}
	fclose(info);

if (exp.x[0]>start) start=exp.x[0];
if (sim.x[0]>start) start=sim.x[0];

if (exp.x[exp.len-1]<stop) stop=exp.x[exp.len-1];
if (sim.x[sim.len-1]<stop) stop=sim.x[sim.len-1];

//inter_chop(&exp,start,stop);
//inter_chop(&sim,start,stop);

double min=inter_get(&exp,start);
inter_sub(&exp,min);
inter_sub(&sim,min);

double max=0.0;

max=inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&sim,1.0, 1.0/max);


error=inter_get_diff(&exp,&sim,out_name,start,stop,config->ivlight_fitmul);

inter_free(&exp);
inter_free(&sim);

return error;
}

double get_charge_ss_error(struct fitvars *config)
{
printf("get ce ss error\n");
double error=0.0;
double max=0.0;
char exp_name[40];
char sim_name[40];
char out_name[40];
strcpy(exp_name,"./exp/cedark/ce.dat");
strcpy(sim_name,"./sim/jv/charge.dat");
strcpy(out_name,"./error_ce.dat");

struct istruct exp;
struct istruct sim;

inter_load_a(&exp,exp_name,0,1);
inter_load_a(&sim,sim_name,0,1);


inter_log_y(&sim);
inter_log_y(&exp);

double start=0.4;
if (sim.x[0]>start) start=sim.x[0];
if (exp.x[0]>start) start=exp.x[0];

double stop=0.6;
if (sim.x[sim.len-1]<stop) stop=sim.x[sim.len-1];
if (exp.x[exp.len-1]<stop) stop=exp.x[exp.len-1];

double min=inter_get(&exp,start);
inter_sub(&exp,min);
inter_sub(&sim,min);



max=inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&sim,1.0, 1.0/max);



FILE *ferror;
ferror=fopen(out_name,"w");


error=inter_get_diff(&exp,&sim,out_name,start,stop,config->chargedark_fitmul);

fclose(ferror);

inter_free(&exp);
inter_free(&sim);

return error;
}

double get_charge_error(struct fitvars *config)
{
printf("get jv error\n");
double error=0.0;
double max=0.0;
struct istruct exp;
struct istruct sim;
char exp_name[40];
char sim_name[40];
char out_name[40];
strcpy(exp_name,"./exp/cedata.dat");
strcpy(sim_name,"./sim/ce/realce.dat");
strcpy(out_name,"./error_ce.dat");

inter_load_a(&exp,exp_name,0,1);
inter_load_a(&sim,sim_name,0,1);
//inter_save(&exp,"./a.dat");
//inter_save(&sim,"./b.dat");
//getchar();
//inter_dump(&sim);

inter_log_y(&sim);
inter_log_y(&exp);

double min=exp.data[0];
inter_sub(&sim,min);
inter_sub(&exp,min);


max=inter_norm_to_one(&exp);
inter_rescale(&sim,1.0, 1.0/max);

double start=exp.x[1];
double stop=exp.x[exp.len-1];

if (sim.x[0]>start) start=sim.x[0];
error=inter_get_diff(&exp,&sim,out_name,start,stop,config->chargedark_fitmul);

inter_free(&exp);
inter_free(&sim);

return error;
}

double get_charge_error_light(struct fitvars *config)
{
printf("get jv error\n");
double error=0.0;
double max=0.0;
struct istruct exp;
struct istruct sim;
char exp_name[40];
char sim_name[40];
char out_name[40];
strcpy(exp_name,"./exp/celight.dat");
strcpy(sim_name,"./sim/celight/realce.dat");
strcpy(out_name,"./error_ce_light.dat");

inter_load_a(&exp,exp_name,0,1);
inter_load_a(&sim,sim_name,0,1);
//inter_save(&exp,"./a.dat");
//inter_save(&sim,"./b.dat");
//getchar();
//inter_dump(&sim);

inter_log_y(&sim);
inter_log_y(&exp);

double start=exp.x[1];
double stop=exp.x[exp.len-1];
double min;

if (sim.x[0]>start) start=sim.x[0];

min=exp.data[0];

inter_sub(&sim,min);
inter_sub(&exp,min);


max=inter_norm_to_one(&exp);
inter_rescale(&sim,1.0, 1.0/max);


error=inter_get_diff(&exp,&sim,out_name,start,stop,config->chargedark_fitmul);

inter_free(&exp);
inter_free(&sim);

return error;
}

double get_ce_trans_error(struct fitvars *config,int ext)
{
//printf("get_ce_trans_error\n");
char exp_name[40];
char sim_name[40];
char config_name[40];
char out_name[40];
char temp[100];
double chop;
sprintf(exp_name,"./exp/ce%d/ce.dat",ext);
sprintf(sim_name,"./sim/ce%d/cei.dat",ext);
sprintf(out_name,"./error_ce_trans%d.dat",ext);
sprintf(config_name,"./exp/ce%d/cefit.inp",ext);

double error=0.0;
double max=0.0;
struct istruct exp;
struct istruct simtime;
double start=-6.7;
double stop=-4;
double error_ce_trans_mull=1.0;

inter_load_a(&exp,exp_name,0,1);
inter_load_a(&simtime,sim_name,0,1);

FILE *file_config;
if ((file_config=fopen(config_name,"r"))==NULL)
{
    printf("Error opening file: %s.\n", strerror(errno));
    exit (0);
}

fscanf(file_config,"%s",temp);
fscanf(file_config,"%le",&(chop));
fscanf(file_config,"%s",temp);
fscanf(file_config,"%le",&(start));
fscanf(file_config,"%s",temp);
fscanf(file_config,"%le",&(stop));
fscanf(file_config,"%s",temp);
fscanf(file_config,"%le",&(error_ce_trans_mull));
fclose(file_config);

inter_chop(&exp,chop, 1.0);
inter_chop(&simtime,chop, 1.0);

//inter_save(&exp,"./a.dat");
//inter_save(&simtime,"./b.dat");

double min=inter_get_min(&simtime);

inter_sub(&simtime,min);
inter_sub(&exp,min);

inter_purge_zero(&simtime);
inter_purge_zero(&exp);

inter_log_y(&simtime);
inter_log_y(&exp);
inter_log_x(&simtime);
inter_log_x(&exp);

inter_chop(&exp,start, stop);
inter_chop(&simtime,start, stop);


min=exp.data[exp.len-1];

inter_sub(&simtime,min);
inter_sub(&exp,min);

start=exp.x[0];
if (start<simtime.x[0]) start=simtime.x[0];
stop=exp.x[exp.len-1];


max=inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&simtime,1.0, 1.0/max);

error=inter_get_diff(&exp,&simtime,out_name,start,stop,error_ce_trans_mull);

inter_free(&exp);
inter_free(&simtime);
//exit(0);
//printf("get_ce_trans_error ret= %d %le %e %e\n",ext,error,min,max );
return error;
}

double get_tpv_error(struct fitvars *config,int ext)
{
char exp_name[40];
char sim_name[40];
char out_name[40];
sprintf(exp_name,"./exp/tpv%d/tpv_voc.dat",ext);
sprintf(sim_name,"./sim/tpv%d/tpv_voc_norm.dat",ext);
sprintf(out_name,"./error_tpv%d.dat",ext);


struct istruct exp;
struct istruct sim;
double start=0.0;
double stop=0.0;
double error=0.0;
double max=0.0;
inter_load_a(&exp,exp_name,0,1);
inter_load_a(&sim,sim_name,0,1);

double min=inter_get_min(&sim);

inter_sub(&sim,min-1e-9);
inter_sub(&exp,min-1e-9);

inter_purge_zero(&sim);
inter_purge_zero(&exp);
min=exp.data[exp.len-1];
inter_save(&exp,"./a.dat");
inter_save(&sim,"./b.dat");
inter_sub(&sim,min);
inter_sub(&exp,min);

start=exp.x[0];
if (start<sim.x[0]) start=sim.x[0];
stop=exp.x[exp.len-1];


max=inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&sim,1.0, 1.0/max);



error=inter_get_diff(&exp,&sim,out_name,start,stop,1.0);

inter_free(&exp);
inter_free(&sim);
printf("TPV returned %le\n",error);
return error;
}

double get_fish_error(struct fitvars *config,int ext)
{
int i;
char exp_name[40];
char sim_name[40];
char sim_np_name[40];
char out_name[40];
sprintf(exp_name,"./exp/fish%d/fish.dat",ext);
sprintf(sim_name,"./sim/fish%d/fish_stark.dat",ext);
sprintf(sim_np_name,"./sim/fish%d/fishnp.dat",ext);
sprintf(out_name,"./error_fish_trans%d.dat",ext);


struct istruct exp;
struct istruct sim;
struct istruct simnp;
struct istruct simnp_deriv;

double start=0.0;
double stop=0.0;
double error=0.0;
double max=0.0;
inter_load_a(&exp,exp_name,0,1);
inter_load_a(&sim,sim_name,0,1);
inter_load_a(&simnp,sim_np_name,0,1);
inter_copy(&simnp_deriv,&simnp,TRUE);
inter_deriv(&simnp_deriv,&simnp);
inter_sort(&exp);

double min=inter_get_min(&sim);

inter_sub(&sim,min);
inter_sub(&exp,min);

inter_purge_zero(&sim);
inter_purge_zero(&exp);
min=exp.data[exp.len-1];

inter_sub(&sim,min);
inter_sub(&exp,min);

start=exp.x[0];
if (start<sim.x[0]) start=sim.x[0];
stop=exp.x[exp.len-1];


max=inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&sim,1.0, 1.0/max);

inter_save(&exp,"./a.dat");
inter_save(&sim,"./b.dat");

error=inter_get_diff(&exp,&sim,out_name,start,stop,1.0);
int neg=FALSE;
for (i=0;i<simnp_deriv.len;i++)
{
	if (simnp_deriv.data[i]<0.0) neg=TRUE;
}
if (neg==TRUE) error+=100.0;
inter_free(&exp);
inter_free(&sim);
inter_free(&simnp_deriv);
inter_free(&simnp);
return error;
}

double get_pulse_error(struct fitvars *config,int ext)
{
char exp_name[40];
char sim_name[40];
char out_name[40];
sprintf(exp_name,"./exp/pulse%d/data.dat",ext);
sprintf(sim_name,"./sim/pulse%d/pulse_j_norm.dat",ext);
sprintf(out_name,"./error_pulse_trans%d.dat",ext);


struct istruct exp;
struct istruct sim;
double start=0.0;
double stop=0.0;
double error=0.0;
double max=0.0;
inter_load_a(&exp,exp_name,0,1);
inter_load_a(&sim,sim_name,0,1);

double min=inter_get_min(&sim);

inter_sub(&sim,min);
inter_sub(&exp,min);

inter_purge_zero(&sim);
inter_purge_zero(&exp);
min=exp.data[exp.len-1];

inter_sub(&sim,min);
inter_sub(&exp,min);

start=exp.x[0];
if (start<sim.x[0]) start=sim.x[0];
stop=exp.x[exp.len-1];


max=inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&sim,1.0, 1.0/max);

inter_save(&exp,"./a.dat");
inter_save(&sim,"./b.dat");

error=inter_get_diff(&exp,&sim,out_name,start,stop,1.0);
printf("My error %le\n",error);
inter_free(&exp);
inter_free(&sim);
return error;
}

double get_fishnp_error(struct fitvars *config,int ext)
{
char exp_name[40];
char sim_name[40];
char out_name[40];
sprintf(exp_name,"./exp/fish%d/fish.dat",ext);
sprintf(sim_name,"./sim/fish%d/fishnp.dat",ext);
sprintf(out_name,"./error_fishnp_trans%d.dat",ext);


struct istruct exp;
struct istruct sim;
double start=0.0;
double stop=0.0;
double error=0.0;
double max=0.0;
inter_load_a(&exp,exp_name,0,1);
inter_load_a(&sim,sim_name,0,1);
inter_sort(&exp);

double min=inter_get_min(&sim);

inter_sub(&sim,min);
inter_sub(&exp,min);

inter_purge_zero(&sim);
inter_purge_zero(&exp);
min=exp.data[0];

inter_sub(&sim,min);
inter_sub(&exp,min);

start=exp.x[0];
if (start<sim.x[0]) start=sim.x[0];
stop=exp.x[exp.len-1];


max=inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&sim,1.0, 1.0/max);

inter_save(&exp,"./a.dat");
inter_save(&sim,"./b.dat");

error=inter_get_diff(&exp,&sim,out_name,start,stop,1.0);

inter_free(&exp);
inter_free(&sim);
return error;
}

double get_tof_error(struct fitvars *config,int ext)
{
printf("get_tof_error\n");
char exp_name[40];
char sim_name[40];
char config_name[40];
char out_name[40];
char temp[100];
double chop;
sprintf(exp_name,"./exp/tof%d/tof.dat",ext);
sprintf(sim_name,"./sim/tof%d/tof.dat",ext);
sprintf(out_name,"./error_tof%d.dat",ext);
sprintf(config_name,"./exp/tof%d/toffit.inp",ext);

double error=0.0;
double max=0.0;
struct istruct exp;
struct istruct simtime;
double start=-6.7;
double stop=-4;

inter_load_a(&exp,exp_name,0,1);
inter_load_a(&simtime,sim_name,0,1);

FILE *file_config;
if ((file_config=fopen(config_name,"r"))==NULL)
{
    printf("Error opening file: %s.\n", strerror(errno));
    exit (0);
}

fscanf(file_config,"%s",temp);
fscanf(file_config,"%le",&(chop));
fscanf(file_config,"%s",temp);
fscanf(file_config,"%le",&(start));
fscanf(file_config,"%s",temp);
fscanf(file_config,"%le",&(stop));


inter_chop(&exp,chop, 1.0);
inter_chop(&simtime,chop, 1.0);


fclose(file_config);

double min=inter_get_min(&simtime);

inter_sub(&simtime,min);
inter_sub(&exp,min);

inter_purge_zero(&simtime);
inter_purge_zero(&exp);

inter_log_y(&simtime);
inter_log_y(&exp);
inter_log_x(&simtime);
inter_log_x(&exp);

inter_chop(&exp,start, stop);
inter_chop(&simtime,start, stop);


min=exp.data[exp.len-1];

inter_sub(&simtime,min);
inter_sub(&exp,min);

start=exp.x[0];
if (start<simtime.x[0]) start=simtime.x[0];
stop=exp.x[exp.len-1];


max=inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&simtime,1.0, 1.0/max);


error=inter_get_diff(&exp,&simtime,out_name,start,stop,config->tof_error_mull);

inter_free(&exp);
inter_free(&simtime);
//exit(0);
printf("get_tof_error ret= %d %e %e %e\n",ext,error,min,max );
return error;
}


double get_tpc_error(struct fitvars *config,int ext)
{
//printf("get_tpc_error\n");
char exp_name[40];
char sim_name[40];
char config_name[40];
char out_name[40];
char temp[100];
sprintf(exp_name,"./exp/tpc%d/tpc.dat",ext);
sprintf(sim_name,"./sim/tpc%d/tpcfit.dat",ext);
sprintf(out_name,"./error_tpc%d.dat",ext);
sprintf(config_name,"./exp/tpc%d/tpcfit.inp",ext);

double error=0.0;
double max=0.0;
struct istruct exp;
struct istruct simtime;
double start=-6.7;
double stop=-4;
double error_mul=1.0;
inter_load_a(&exp,exp_name,0,1);
inter_load_a(&simtime,sim_name,0,1);

inter_chop(&exp,1e-7, 1.0);
inter_chop(&simtime,1e-7, 1.0);

FILE *file_config;
if ((file_config=fopen(config_name,"r"))==NULL)
{
    printf("Error opening file: %s.\n", strerror(errno));
    exit (0);
}

fscanf(file_config,"%s",temp);
fscanf(file_config,"%le",&(start));
fscanf(file_config,"%s",temp);
fscanf(file_config,"%le",&(stop));
fscanf(file_config,"%s",temp);
fscanf(file_config,"%le",&(error_mul));
fclose(file_config);

double min=inter_get_min(&simtime);

inter_sub(&simtime,min);
inter_sub(&exp,min);

inter_purge_zero(&simtime);
inter_purge_zero(&exp);

inter_log_y(&simtime);
inter_log_y(&exp);
inter_log_x(&simtime);
inter_log_x(&exp);

inter_chop(&exp,start, stop);
inter_chop(&simtime,start, stop);

min=exp.data[exp.len-1];

inter_sub(&simtime,min);
inter_sub(&exp,min);

start=exp.x[0];
if (start<simtime.x[0]) start=simtime.x[0];
stop=exp.x[exp.len-1];


max=inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&simtime,1.0, 1.0/max);

//inter_save(&exp,"./a.dat");
//inter_save(&simtime,"./b.dat");

error=inter_get_diff(&exp,&simtime,out_name,start,stop,error_mul);

inter_free(&exp);
inter_free(&simtime);
//exit(0);
//printf("get_tpc_error ret= %d %e %e %e\n",ext,error,min,max );
return error;
}

double get_jv_error(struct fitvars *config)
{
printf("get_jv_error\n");
char expfile[1000];
char simfile[1000];
char out_name[1000];
strcpy(expfile,"./exp/jv/jv.dat");
strcpy(simfile,"./sim/jv/jvexternal.dat");
sprintf(out_name,"./error_jv_dark.dat");

double error=0.0;
struct istruct exp;
struct istruct sim;
inter_load_a(&exp,expfile,0,1);
inter_load_a(&sim,simfile,0,1);

double max=0.0;
char tempstring[100];
double start=0.0;
double stop=0.8;

FILE *info=fopena("./exp/jv/","fit.inp","r");
if (info==NULL)
{
	printf("Problem with ./exp/jv/fit.inp\n");
	exit(0);
}
fscanf(info,"%s",tempstring);
fscanf(info,"%le",&start);
fscanf(info,"%s",tempstring);
fscanf(info,"%le",&stop);
fscanf(info,"%s",tempstring);

if (strcmp("#end",tempstring)!=0)
{
	printf("Problem with fit.inp in %s!\n","./exp/jv/");
	exit(0);
}
fclose(info);

if (sim.x[0]>start) start=sim.x[0];
if (exp.x[0]>start) start=exp.x[0];



inter_log_y(&sim);
inter_log_y(&exp);

double min=inter_get(&exp,start);
inter_sub(&exp,min);
inter_sub(&sim,min);

max=inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&sim,1.0, 1.0/max);



error=inter_get_diff(&exp,&sim,out_name,start,stop,config->ivdark_fitmul);

inter_free(&exp);
inter_free(&sim);
return error;
}

double get_band_error(struct fitvars *config)
{
int dostype=read_value("dos.inp",0,2);
double Ee=0.0;
double Eh=0.0;
double ret=0.0;
if (dostype==0)
{
	Ee=read_value("dos.inp",0,38);
	Eh=read_value("dos.inp",0,40);
	ret=1+tanh( (Ee-Eh)*20.0/Ee);
	ret*=config->banderror_fitmul;
}else
{

	Ee=read_value("dos.inp",0,12);
	Eh=read_value("dos.inp",0,14);
	ret=1+tanh( (Ee-Eh)*10.0/Ee);
	ret*=config->banderror_fitmul;
}

return ret;
}

