// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "sim.h"

char *exp_to_latex(double in)
{
int pos=0;
char temp[100];
static char out[100];
sprintf(temp,"%.3e",in);
int i;
for (i=0;i<strlen(temp);i++)
{

if (temp[i]!='+')
{
	if (temp[i]=='e')
	{
	out[pos++]='\\';
	out[pos++]=temp[i];
	out[pos++]='{';
	}else
	{
	out[pos++]=temp[i];
	}
}

}
out[pos++]='}';
out[pos++]=0;
return out;
}

void dump_talbe()
{
FILE *out;
out = fopen("./tab.tex","w");
double n=read_value("device.inp",0,11);
double p=read_value("device.inp",0,15);
double xi=3.8;
double eg=1.1;
double temp=n;
if (p<0)
{
n=p;
p=temp;
}
n=fabs(n);
p=fabs(p);
fprintf(out,"Left contact electron density & \\begin{math}n_l\\end{math} & %s\\begin{math}m^{-3}\\end{math} \\\\\n",exp_to_latex(n));
fprintf(out,"Right contact hole density  & \\begin{math}p_r\\end{math} & \\begin{math}%s\\end{math} \\begin{math}m^{-3}\\end{math} \\\\\n",exp_to_latex(p));
fprintf(out,"Effective electron trap density  & \\begin{math}N^{exp}_e\\end{math} & %s \\begin{math}m^{-3}{eV^{-1}}\\end{math}   \\\\\n",exp_to_latex(read_value("dos.inp",0,8)));
fprintf(out,"Effective hole trap density  & \\begin{math}N^{exp}_h\\end{math} & %s \\begin{math}m^{-3}{eV^{-1}}\\end{math} \\\\\n",exp_to_latex(read_value("dos.inp",0,10)));
fprintf(out,"Characteristic energy for electron exponential tail& \\begin{math}E^u_e\\end{math} & %.0lf meV\\\\\n",read_value("dos.inp",0,12)*1000.0);
fprintf(out,"Characteristic energy for hole exponential tail& \\begin{math}E^u_h\\end{math} & %.0lf meV\\\\\n",read_value("dos.inp",0,14)*1000.0);
fprintf(out,"Effective density of free electron states & \\begin{math}N_{c}\\end{math} & %s \\begin{math}m^{-3}\\end{math}\\\\\n",exp_to_latex(read_value("dos.inp",0,64)));
fprintf(out,"Effective density of free hole states & \\begin{math}N_{v}\\end{math} & %s \\begin{math}m^{-3}\\end{math}\\\\\n",exp_to_latex(read_value("dos.inp",0,66)));

fprintf(out,"LUMO electron capture cross section & \\begin{math}\\sigma^e_e\\end{math} & %s \\begin{math}m^{-2}\\end{math} \\\\\n",exp_to_latex(read_value("dos.inp",0,52)));
fprintf(out,"LUMO hole capture cross section & \\begin{math}\\sigma^e_h\\end{math} & %s \\begin{math}m^{-2}\\end{math} \\\\\n",exp_to_latex(read_value("dos.inp",0,54)));
fprintf(out,"HOMO electron capture cross section & \\begin{math}\\sigma^h_e\\end{math} & %s \\begin{math}m^{-2}\\end{math} \\\\\n",exp_to_latex(read_value("dos.inp",0,58)));
fprintf(out,"HOMO hole capture cross section & \\begin{math}\\sigma^h_h\\end{math} & %s \\begin{math}m^{-2}\\end{math} \\\\\n",exp_to_latex(read_value("dos.inp",0,60)));

fprintf(out,"Free electron mobility & \\begin{math}\\mu_{e0}\\end{math} & %s \\begin{math}m^{2} V^{-1} s^{-1}\\end{math}\\\\\n",exp_to_latex(read_value("dos.inp",0,18)));
fprintf(out,"Free hole mobility& \\begin{math}\\mu_{h0}\\end{math} & %s \\begin{math}m^{2} V^{-1} s^{-1}\\end{math}\\\\\n",exp_to_latex(read_value("dos.inp",0,20)));
fprintf(out,"\n");
fprintf(out,"Shunt resistance & \\begin{math}R_{shunt}\\end{math} & %s \\begin{math} \\Omega\\end{math}\\\\\n",exp_to_latex(read_value("device.inp",0,25)));
fprintf(out,"Contact resistance  & \\begin{math}R_{contact}\\end{math} & %.1lf \\begin{math} \\Omega cm^{-2}\\end{math}  \\\\\n",read_value("device.inp",0,27));
fprintf(out,"Thermal velocity   & \\begin{math}v_{th}\\end{math} & %s \\begin{math}m s^{-1}\\end{math} \\\\\n",exp_to_latex(read_value("dos.inp",0,56)));
fprintf(out,"LUMO mobility edge & \\begin{math}E^d_e\\end{math} & %.3lf eV\\\\\n",-xi);
fprintf(out,"HOMO mobility edge & \\begin{math}E^d_h\\end{math} & %.3lf eV\\\\\n",-xi-eg);


fclose(out);
}
